;-------------------------------------------------------------------------------
;
;  Mem.inc
;  programmed by Marco Kaufmann
;  Release 2002/09/16
;
;  Include file for CodeX Assembler programs
;
;  Mem.inc provides support of memory filling and moving routines.
;  Mem.inc is Non-OS dependend and works on every system
;
;  This file as part of the CodeX Assembler package is distributed free,
;  but without any warranty, even without the warranty of merchantability.
;  I remind, that use is on own risk.
;
;-------------------------------------------------------------------------------


;---Pseudo interface------------------------------------------------------------
;   These are the routines provided by mem.inc
;
;
;   fillchar
;   fillword
;   filldword
;   fillcharfar
;   fillwordfar
;   filldwordfar
;   move
;   moveup
;   movedown
;   movefar
;   moveupfar
;   movedownfar



;---FillChar--------------------------------------------------------------------
;   param : eax = offset buff, ecx = bytes, bl = val
;   return: none


fillchar:
  mov bh,bl             ;weiter mit fillword


;---FillWord--------------------------------------------------------------------
;   param : eax = offset buff, ecx = bytes, bx = val
;   return:


fillword:
  mov dx,bx
  shl edx,16
  mov dx,bx
  mov ebx,edx           ;weiter mit filldword


;---FillDword-------------------------------------------------------------------
;   param : eax = offset buff, ecx = bytes, ebx = val
;   return:


filldword:

  test ecx,1
  jz @mfilldword1
  mov [eax+ecx-1],bl
  @mfilldword1:
  shr ecx,1

  test ecx,1
  jz @mfilldword2
  mov [eax+ecx*2-2],bx
  @mfilldword2:
  shr ecx,1

  @mfilldword3:
  mov [eax+ecx*4-4],ebx
  dec ecx
  jnz @mfilldword3
  ret


;---FillCharFar-----------------------------------------------------------------
;   param : eax = offset buff, ecx: bytes, bl = val, gs = buffseg
;   return: none


fillcharfar:
  mov bh,bl             ;weiter mit fillwordfar


;---FillWordFar-----------------------------------------------------------------
;   param : eax = offset buff, ecx = bytes, bx = val, gs = buffseg
;   return:

fillwordfar:
  mov dx,bx
  shl edx,16
  mov dx,bx
  mov ebx,edx           ;weiter mit filldwordfar


;---FillDwordFar----------------------------------------------------------------
;   param : eax = offset buff, ecx = bytes, ebx: val, gs = buffseg
;   return: none


filldwordfar:

  test ecx,1
  jz @mfilldwordfar1
  mov gs:[eax+ecx-1],bl
  @mfilldwordfar1:
  shr ecx,1

  test ecx,1
  jz @mfilldwordfar2
  mov gs:[eax+ecx*2-2],bx
  @mfilldwordfar2:
  shr ecx,1

  @mfilldwordfar3:
  mov gs:[eax+ecx*4-4],ebx
  dec ecx
  jnz @mfilldwordfar3
  ret


;---Move------------------------------------------------------------------------
;   param : eax = offset dst, ebx = offset src, ecx = bytes
;   return: none


move:

  test ecx,1
  jz @mmove1
  mov edx,[ebx+ecx-1]
  mov [eax+ecx-1],edx
  @mmove1:
  shr ecx,1

  test ecx,1
  jz @mmove2
  mov edx,[ebx+ecx*2-2]
  mov [eax+ecx*2-2],edx
  @mmove2:
  shr ecx,1

  @mmove3:
  mov edx,[ebx+ecx*4-4]
  mov [eax+ecx*4-4],edx
  dec ecx
  jnz @mmove3
  ret


;---MoveFar---------------------------------------------------------------------
;   param : eax = offset dst, ebx = offset src, ecx = bytes, gs: dstseg, fs = srcseg
;   return: none


movefar:

  test ecx,1
  jz @mmovefar1
  mov edx,fs:[ebx+ecx-1]
  mov gs:[eax+ecx-1],edx
  @mmovefar1:
  shr ecx,1

  test ecx,1
  jz @mmovefar2
  mov edx,fs:[ebx+ecx*2-2]
  mov gs:[eax+ecx*2-2],edx
  @mmovefar2:
  shr ecx,1

  @mmovefar3:
  mov edx,fs:[ebx+ecx*4-4]
  mov gs:[eax+ecx*4-4],edx
  dec ecx
  jnz @mmovefar3
  ret